!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center
! Land Information System Framework (LISF)
! Version 7.4
!
! Copyright (c) 2022 United States Government as represented by the
! Administrator of the National Aeronautics and Space Administration.
! All Rights Reserved.
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
#include "LDT_misc.h"
!BOP
! !ROUTINE: read_WRFAK_elev
! \label{read_WRFAK_elev}
!
! !REVISION HISTORY:
!
!  22 Sep 2016: James Geiger; Initial Specification
!  21 Jun 2021: K.R. Arsenault; Updated for different WRF output files
!
! !INTERFACE:
subroutine read_WRFAK_elev(n,findex, wrfakelev, elevdiff )

! !USES:
  use LDT_coreMod,       only : LDT_rc, LDT_domain
  use LDT_metforcingMod, only : LDT_forc
  use LDT_fileIOMod,     only : LDT_read_param
  use LDT_logMod,        only : LDT_logunit, LDT_endrun, LDT_verify
  use LDT_fileIOMod,     only : LDT_transform_paramgrid
  use WRF_AKdom_forcingMod, only : WRFAK_struc
#if (defined USE_NETCDF3 || defined USE_NETCDF4) 
  use netcdf
#endif

  implicit none

! !ARGUMENTS: 
  integer, intent(in) :: n
  integer, intent(in) :: findex
!- Terrain height will be set to run domain:
  real, intent(inout) :: wrfakelev(LDT_rc%lnc(n),LDT_rc%lnr(n),1)
  real, intent(inout) :: elevdiff(LDT_rc%met_nc(findex),LDT_rc%met_nr(findex))

! !DESCRIPTION:
!
!  Opens and reads WRF-Alaska 4km model elevation to the LDT grid.
!  The data will be used to perform any topographical adjustments
!  to the forcing.
!
!  The elevation file needs to be preprocessed to fit the running 
!  resolution and domain.
!
!  The arguments are: 
!  \begin{description}
!  \item[n]
!   index of the nest
!  \item[findex]
!   index of the forcing source
!  \item[wrfakelev]
!   WRF forcing terrain height map
!  \item[elevdiff]
!   Placeholder field if an elevation difference map is needed 
!  \end{description}
!
!EOP
   logical :: file_exists
   integer :: ftn_const
   integer :: i,c,r,k,iret

   integer :: hgtId
   real    :: read_hgt(WRFAK_struc(n)%nc, WRFAK_struc(n)%nr,1)

   ! Grid transform fields:
   integer   :: inpts, outpts
   real      :: elev1d(WRFAK_struc(n)%nc*WRFAK_struc(n)%nr)
   logical*1 :: lb(WRFAK_struc(n)%nc*WRFAK_struc(n)%nr)
   real      :: elev_regrid(LDT_rc%lnc(n)*LDT_rc%lnr(n))
   logical*1 :: lb_regrid(LDT_rc%lnc(n)*LDT_rc%lnr(n))
! _____________________________________________________________________________

#if (defined USE_NETCDF3) 
  write(LDT_logunit,*) "[ERR] WRF Alaska terrain height reader requires NetCDF4"
  call LDT_endrun()
#endif

   wrfakelev = LDT_rc%udef
   elevdiff = LDT_rc%udef

   ! Check if WRF-AK grid is selected but downscaling options, 
   !  like bilinear, is incorrectly selected.

   if( WRFAK_struc(n)%gridDesci(9)  == LDT_rc%gridDesc(n,9) .and. &
       WRFAK_struc(n)%gridDesci(10) == LDT_rc%gridDesc(n,10).and. &
       LDT_rc%gridDesc(n,1) == 0 .and. &
       LDT_rc%met_gridtransform_parms(findex) .ne. "neighbor" ) then

      write(LDT_logunit,*) "[ERR] The WRF-Alaska grid was selected for the"
      write(LDT_logunit,*) "  LDT run domain; however, 'bilinear', 'budget-bilinear',"
      write(LDT_logunit,*) "  or some other unknown option was selected to spatially"
      write(LDT_logunit,*) "  downscale the grid, which will cause errors during runtime."
      write(LDT_logunit,*) "Program stopping ..."
      call LDT_endrun()
   endif

   inquire(file = trim(WRFAK_struc(n)%file_wrfelev), exist=file_exists)
   if(.not. file_exists) then
      write(LDT_logunit,*) "[ERR] The WRF-Alaska terrain height file ",&
            trim(WRFAK_struc(n)%file_wrfelev)," is not found."
      write(LDT_logunit,*) "Program stopping ..."
      call LDT_endrun
   endif

! -------------------------------------------------------------------
! Open and Read-in Forcing Terrain Hght File - Bring to LIS run domain
! -------------------------------------------------------------------
   write(LDT_logunit,*) "[INFO] Reading the WRF-Alaska terrain height file: ", &
        WRFAK_struc(n)%file_wrfelev

#if (defined USE_NETCDF4) 

   ! Open the WRF-AK "constants" file to read in the 
   !  height field (HGT):
   call LDT_verify(nf90_open(path=trim(WRFAK_struc(n)%file_wrfelev), &
            mode=NF90_NOWRITE, ncid=ftn_const), &
           'nf90_open failed in read_WRFAK_elev')

   call LDT_verify(nf90_inq_varid(ftn_const,'HGT_M',hgtId), &
           'nf90_inq_varid failed for HGT in read_WRFAK_elev')

   ! Reading in HGT field:
   call LDT_verify(nf90_get_var(ftn_const, hgtId, read_hgt), &
           'nf90_get_var failed for HGT in read_WRFAK_elev')


   ! Initialize arrays for grid transformation:
   inpts  = LDT_rc%met_nc(findex)*LDT_rc%met_nr(findex)
   outpts = LDT_rc%lnr(n)*LDT_rc%lnc(n)
   lb     = .true.
   lb_regrid = .true.
   elev1d = 1.e+15

   ! Convert 2D to 1D array for the interplation call:
   do r = 1, LDT_rc%met_nr(findex)
      do c = 1, LDT_rc%met_nc(findex)
         k= c+(r-1)*LDT_rc%met_nc(findex)
         elev1d(k) = read_hgt(c,r,1)
         if ( elev1d(k) == 1.e+15 ) then
           elev1d(k)  = LDT_rc%udef
           lb(k) = .false.
         endif
      enddo
   enddo

   ! Interp elevation field to output field:
   call LDT_transform_paramgrid(n, LDT_rc%met_gridtransform_parms(findex), &
            LDT_rc%met_gridDesc(findex,:), inpts, 1, elev1d, lb, &
            outpts, elev_regrid, lb_regrid )

   ! Convert 1D to 2D height to terrain height in meters:
   i = 0
   do r = 1, LDT_rc%lnr(n)
      do c = 1, LDT_rc%lnc(n)
         i = i + 1
         wrfakelev(c,r,1) = elev_regrid(i)
      end do
   end do

#endif
end subroutine read_WRFAK_elev
